@@ -28,12 +28,18 @@ pub enum TargetTriple {
     #[serde(alias = "x8664-pc-windows-msvc")]
     X8664PcWindowsMsvc,
 
+    /// An ARM64 Windows target.
+    #[cfg_attr(feature = "clap", value(name = "aarch64-pc-windows-msvc"))]
+    #[serde(rename = "aarch64-pc-windows-msvc")]
+    #[serde(alias = "arm64-pc-windows-msvc")]
+    Aarch64PcWindowsMsvc,
+
     /// A 32-bit x86 Windows target.
     #[cfg_attr(feature = "clap", value(name = "i686-pc-windows-msvc"))]
     #[serde(rename = "i686-pc-windows-msvc")]
     I686PcWindowsMsvc,
 
-    /// An x86 Linux target. Equivalent to `x86_64-manylinux_2_17`.
+    /// An x86 Linux target. Equivalent to `x86_64-manylinux_2_28`.
     #[cfg_attr(feature = "clap", value(name = "x86_64-unknown-linux-gnu"))]
     #[serde(rename = "x86_64-unknown-linux-gnu")]
     #[serde(alias = "x8664-unknown-linux-gnu")]
@@ -56,7 +62,7 @@ pub enum TargetTriple {
     #[serde(alias = "x8664-apple-darwin")]
     X8664AppleDarwin,
 
-    /// An ARM64 Linux target. Equivalent to `aarch64-manylinux_2_17`.
+    /// An ARM64 Linux target. Equivalent to `aarch64-manylinux_2_28`.
     #[cfg_attr(feature = "clap", value(name = "aarch64-unknown-linux-gnu"))]
     #[serde(rename = "aarch64-unknown-linux-gnu")]
     Aarch64UnknownLinuxGnu,
@@ -72,6 +78,11 @@ pub enum TargetTriple {
     #[serde(alias = "x8664-unknown-linux-musl")]
     X8664UnknownLinuxMusl,
 
+    /// A RISCV64 Linux target.
+    #[cfg_attr(feature = "clap", value(name = "riscv64-unknown-linux"))]
+    #[serde(rename = "riscv64-unknown-linux")]
+    Riscv64UnknownLinuxGnu,
+
     /// An `x86_64` target for the `manylinux2014` platform. Equivalent to `x86_64-manylinux_2_17`.
     #[cfg_attr(feature = "clap", value(name = "x86_64-manylinux2014"))]
     #[serde(rename = "x86_64-manylinux2014")]
@@ -237,10 +248,11 @@ impl TargetTriple {
     pub fn platform(self) -> Platform {
         match self {
             Self::Windows | Self::X8664PcWindowsMsvc => Platform::new(Os::Windows, Arch::X86_64),
+            Self::Aarch64PcWindowsMsvc => Platform::new(Os::Windows, Arch::Aarch64),
             Self::Linux | Self::X8664UnknownLinuxGnu => Platform::new(
                 Os::Manylinux {
                     major: 2,
-                    minor: 17,
+                    minor: 28,
                 },
                 Arch::X86_64,
             ),
@@ -262,10 +274,17 @@ impl TargetTriple {
             Self::Aarch64UnknownLinuxGnu => Platform::new(
                 Os::Manylinux {
                     major: 2,
-                    minor: 17,
+                    minor: 28,
                 },
                 Arch::Aarch64,
             ),
+            Self::Riscv64UnknownLinuxGnu => Platform::new(
+                Os::Manylinux {
+                    major: 2,
+                    minor: 39,
+                },
+                Arch::Riscv64,
+            ),
             Self::Aarch64UnknownLinuxMusl => {
                 Platform::new(Os::Musllinux { major: 1, minor: 2 }, Arch::Aarch64)
             }
@@ -468,6 +487,7 @@ impl TargetTriple {
     pub fn platform_machine(self) -> &'static str {
         match self {
             Self::Windows | Self::X8664PcWindowsMsvc => "x86_64",
+            Self::Aarch64PcWindowsMsvc => "ARM64",
             Self::Linux | Self::X8664UnknownLinuxGnu => "x86_64",
             Self::Macos | Self::Aarch64AppleDarwin => "arm64",
             Self::I686PcWindowsMsvc => "x86",
@@ -475,6 +495,7 @@ impl TargetTriple {
             Self::Aarch64UnknownLinuxGnu => "aarch64",
             Self::Aarch64UnknownLinuxMusl => "aarch64",
             Self::X8664UnknownLinuxMusl => "x86_64",
+            Self::Riscv64UnknownLinuxGnu => "riscv64",
             Self::X8664Manylinux2014 => "x86_64",
             Self::X8664Manylinux217 => "x86_64",
             Self::X8664Manylinux228 => "x86_64",
@@ -509,6 +530,7 @@ impl TargetTriple {
     pub fn platform_system(self) -> &'static str {
         match self {
             Self::Windows | Self::X8664PcWindowsMsvc => "Windows",
+            Self::Aarch64PcWindowsMsvc => "Windows",
             Self::Linux | Self::X8664UnknownLinuxGnu => "Linux",
             Self::Macos | Self::Aarch64AppleDarwin => "Darwin",
             Self::I686PcWindowsMsvc => "Windows",
@@ -516,6 +538,7 @@ impl TargetTriple {
             Self::Aarch64UnknownLinuxGnu => "Linux",
             Self::Aarch64UnknownLinuxMusl => "Linux",
             Self::X8664UnknownLinuxMusl => "Linux",
+            Self::Riscv64UnknownLinuxGnu => "Linux",
             Self::X8664Manylinux2014 => "Linux",
             Self::X8664Manylinux217 => "Linux",
             Self::X8664Manylinux228 => "Linux",
@@ -550,6 +573,7 @@ impl TargetTriple {
     pub fn platform_version(self) -> &'static str {
         match self {
             Self::Windows | Self::X8664PcWindowsMsvc => "",
+            Self::Aarch64PcWindowsMsvc => "",
             Self::Linux | Self::X8664UnknownLinuxGnu => "",
             Self::Macos | Self::Aarch64AppleDarwin => "",
             Self::I686PcWindowsMsvc => "",
@@ -557,6 +581,7 @@ impl TargetTriple {
             Self::Aarch64UnknownLinuxGnu => "",
             Self::Aarch64UnknownLinuxMusl => "",
             Self::X8664UnknownLinuxMusl => "",
+            Self::Riscv64UnknownLinuxGnu => "",
             Self::X8664Manylinux2014 => "",
             Self::X8664Manylinux217 => "",
             Self::X8664Manylinux228 => "",
@@ -594,6 +619,7 @@ impl TargetTriple {
     pub fn platform_release(self) -> &'static str {
         match self {
             Self::Windows | Self::X8664PcWindowsMsvc => "",
+            Self::Aarch64PcWindowsMsvc => "",
             Self::Linux | Self::X8664UnknownLinuxGnu => "",
             Self::Macos | Self::Aarch64AppleDarwin => "",
             Self::I686PcWindowsMsvc => "",
@@ -601,6 +627,7 @@ impl TargetTriple {
             Self::Aarch64UnknownLinuxGnu => "",
             Self::Aarch64UnknownLinuxMusl => "",
             Self::X8664UnknownLinuxMusl => "",
+            Self::Riscv64UnknownLinuxGnu => "",
             Self::X8664Manylinux2014 => "",
             Self::X8664Manylinux217 => "",
             Self::X8664Manylinux228 => "",
@@ -637,6 +664,7 @@ impl TargetTriple {
     pub fn os_name(self) -> &'static str {
         match self {
             Self::Windows | Self::X8664PcWindowsMsvc => "nt",
+            Self::Aarch64PcWindowsMsvc => "nt",
             Self::Linux | Self::X8664UnknownLinuxGnu => "posix",
             Self::Macos | Self::Aarch64AppleDarwin => "posix",
             Self::I686PcWindowsMsvc => "nt",
@@ -644,6 +672,7 @@ impl TargetTriple {
             Self::Aarch64UnknownLinuxGnu => "posix",
             Self::Aarch64UnknownLinuxMusl => "posix",
             Self::X8664UnknownLinuxMusl => "posix",
+            Self::Riscv64UnknownLinuxGnu => "posix",
             Self::X8664Manylinux2014 => "posix",
             Self::X8664Manylinux217 => "posix",
             Self::X8664Manylinux228 => "posix",
@@ -678,6 +707,7 @@ impl TargetTriple {
     pub fn sys_platform(self) -> &'static str {
         match self {
             Self::Windows | Self::X8664PcWindowsMsvc => "win32",
+            Self::Aarch64PcWindowsMsvc => "win32",
             Self::Linux | Self::X8664UnknownLinuxGnu => "linux",
             Self::Macos | Self::Aarch64AppleDarwin => "darwin",
             Self::I686PcWindowsMsvc => "win32",
@@ -685,6 +715,7 @@ impl TargetTriple {
             Self::Aarch64UnknownLinuxGnu => "linux",
             Self::Aarch64UnknownLinuxMusl => "linux",
             Self::X8664UnknownLinuxMusl => "linux",
+            Self::Riscv64UnknownLinuxGnu => "linux",
             Self::X8664Manylinux2014 => "linux",
             Self::X8664Manylinux217 => "linux",
             Self::X8664Manylinux228 => "linux",
@@ -719,6 +750,7 @@ impl TargetTriple {
     pub fn manylinux_compatible(self) -> bool {
         match self {
             Self::Windows | Self::X8664PcWindowsMsvc => false,
+            Self::Aarch64PcWindowsMsvc => false,
             Self::Linux | Self::X8664UnknownLinuxGnu => true,
             Self::Macos | Self::Aarch64AppleDarwin => false,
             Self::I686PcWindowsMsvc => false,
@@ -726,6 +758,7 @@ impl TargetTriple {
             Self::Aarch64UnknownLinuxGnu => true,
             Self::Aarch64UnknownLinuxMusl => true,
             Self::X8664UnknownLinuxMusl => true,
+            Self::Riscv64UnknownLinuxGnu => true,
             Self::X8664Manylinux2014 => true,
             Self::X8664Manylinux217 => true,
             Self::X8664Manylinux228 => true,
