@@ -1,4 +1,5 @@
 use core::fmt;
+use std::borrow::Cow;
 use std::cmp::Reverse;
 use std::ffi::OsStr;
 use std::io::{self, Write};
@@ -12,13 +13,13 @@ use itertools::Itertools;
 use same_file::is_same_file;
 use thiserror::Error;
 use tracing::{debug, warn};
-use uv_configuration::{Preview, PreviewFeatures};
+use uv_preview::{Preview, PreviewFeatures};
 #[cfg(windows)]
 use windows_sys::Win32::Storage::FileSystem::FILE_ATTRIBUTE_REPARSE_POINT;
 
 use uv_fs::{LockedFile, Simplified, replace_symlink, symlink_or_copy_file};
-use uv_platform::Error as PlatformError;
-use uv_platform::{Arch, Libc, LibcDetectionError, Os};
+use uv_platform::{Error as PlatformError, Os};
+use uv_platform::{LibcDetectionError, Platform};
 use uv_state::{StateBucket, StateStore};
 use uv_static::EnvVars;
 use uv_trampoline_builder::{Launcher, windows_python_launcher};
@@ -259,19 +260,16 @@ impl ManagedPythonInstallations {
     pub fn find_matching_current_platform(
         &self,
     ) -> Result<impl DoubleEndedIterator<Item = ManagedPythonInstallation> + use<>, Error> {
-        let os = Os::from_env();
-        let arch = Arch::from_env();
-        let libc = Libc::from_env()?;
+        let platform = Platform::from_env()?;
 
-        let iter = ManagedPythonInstallations::from_settings(None)?
+        let iter = Self::from_settings(None)?
             .find_all()?
             .filter(move |installation| {
-                installation.key.os == os
-                    && (arch.supports(installation.key.arch)
-                        // TODO(zanieb): Allow inequal variants, as `Arch::supports` does not
-                        // implement this yet. See https://github.com/astral-sh/uv/pull/9788
-                        || arch.family() == installation.key.arch.family())
-                    && installation.key.libc == libc
+                if !platform.supports(installation.platform()) {
+                    debug!("Skipping managed installation `{installation}`: not supported by current platform `{platform}`");
+                    return false;
+                }
+                true
             });
 
         Ok(iter)
@@ -317,11 +315,15 @@ pub struct ManagedPythonInstallation {
     /// The URL with the Python archive.
     ///
     /// Empty when self was constructed from a path.
-    url: Option<&'static str>,
+    url: Option<Cow<'static, str>>,
     /// The SHA256 of the Python archive at the URL.
     ///
     /// Empty when self was constructed from a path.
-    sha256: Option<&'static str>,
+    sha256: Option<Cow<'static, str>>,
+    /// The build version of the Python installation.
+    ///
+    /// Empty when self was constructed from a path without a BUILD file.
+    build: Option<Cow<'static, str>>,
 }
 
 impl ManagedPythonInstallation {
@@ -329,8 +331,9 @@ impl ManagedPythonInstallation {
         Self {
             path,
             key: download.key().clone(),
-            url: Some(download.url()),
-            sha256: download.sha256(),
+            url: Some(download.url().clone()),
+            sha256: download.sha256().cloned(),
+            build: download.build().map(Cow::Borrowed),
         }
     }
 
@@ -344,11 +347,19 @@ impl ManagedPythonInstallation {
 
         let path = std::path::absolute(&path).map_err(|err| Error::AbsolutePath(path, err))?;
 
+        // Try to read the BUILD file if it exists
+        let build = match fs::read_to_string(path.join("BUILD")) {
+            Ok(content) => Some(Cow::Owned(content.trim().to_string())),
+            Err(err) if err.kind() == io::ErrorKind::NotFound => None,
+            Err(err) => return Err(err.into()),
+        };
+
         Ok(Self {
             path,
             key,
             url: None,
             sha256: None,
+            build,
         })
     }
 
@@ -361,8 +372,6 @@ impl ManagedPythonInstallation {
     /// If windowed is true, `pythonw.exe` is selected over `python.exe` on windows, with no changes
     /// on non-windows.
     pub fn executable(&self, windowed: bool) -> PathBuf {
-        let implementation = self.implementation().executable_name();
-
         let version = match self.implementation() {
             ImplementationName::CPython => {
                 if cfg!(unix) {
@@ -373,12 +382,14 @@ impl ManagedPythonInstallation {
             }
             // PyPy uses a full version number, even on Windows.
             ImplementationName::PyPy => format!("{}.{}", self.key.major, self.key.minor),
+            // Pyodide and GraalPy do not have a version suffix.
+            ImplementationName::Pyodide => String::new(),
             ImplementationName::GraalPy => String::new(),
         };
 
         // On Windows, the executable is just `python.exe` even for alternative variants
         // GraalPy always uses `graalpy.exe` as the main executable
-        let variant = if *self.implementation() == ImplementationName::GraalPy {
+        let variant = if self.implementation() == ImplementationName::GraalPy {
             ""
         } else if cfg!(unix) {
             self.key.variant.suffix()
@@ -391,13 +402,15 @@ impl ManagedPythonInstallation {
 
         let name = format!(
             "{implementation}{version}{variant}{exe}",
+            implementation = self.implementation().executable_name(),
             exe = std::env::consts::EXE_SUFFIX
         );
 
         let executable = executable_path_from_base(
             self.python_dir().as_path(),
             &name,
-            &LenientImplementationName::from(*self.implementation()),
+            &LenientImplementationName::from(self.implementation()),
+            *self.key.os(),
         );
 
         // Workaround for python-build-standalone v20241016 which is missing the standard
@@ -434,8 +447,8 @@ impl ManagedPythonInstallation {
         self.key.version()
     }
 
-    pub fn implementation(&self) -> &ImplementationName {
-        match self.key.implementation() {
+    pub fn implementation(&self) -> ImplementationName {
+        match self.key.implementation().into_owned() {
             LenientImplementationName::Known(implementation) => implementation,
             LenientImplementationName::Unknown(_) => {
                 panic!("Managed Python installations should have a known implementation")
@@ -451,6 +464,15 @@ impl ManagedPythonInstallation {
         &self.key
     }
 
+    pub fn platform(&self) -> &Platform {
+        self.key.platform()
+    }
+
+    /// The build version of this installation, if available.
+    pub fn build(&self) -> Option<&str> {
+        self.build.as_deref()
+    }
+
     pub fn minor_version_key(&self) -> &PythonInstallationMinorVersionKey {
         PythonInstallationMinorVersionKey::ref_cast(&self.key)
     }
@@ -465,10 +487,10 @@ impl ManagedPythonInstallation {
                 .file_name()
                 .is_some_and(|filename| filename.to_string_lossy() == *name),
             PythonRequest::Implementation(implementation) => {
-                implementation == self.implementation()
+                *implementation == self.implementation()
             }
             PythonRequest::ImplementationVersion(implementation, version) => {
-                implementation == self.implementation() && version.matches_version(&self.version())
+                *implementation == self.implementation() && version.matches_version(&self.version())
             }
             PythonRequest::Version(version) => version.matches_version(&self.version()),
             PythonRequest::Key(request) => request.satisfied_by_key(self.key()),
@@ -543,8 +565,13 @@ impl ManagedPythonInstallation {
     /// Ensure the environment is marked as externally managed with the
     /// standard `EXTERNALLY-MANAGED` file.
     pub fn ensure_externally_managed(&self) -> Result<(), Error> {
+        if self.key.os().is_emscripten() {
+            // Emscripten's stdlib is a zip file so we can't put an
+            // EXTERNALLY-MANAGED inside.
+            return Ok(());
+        }
         // Construct the path to the `stdlib` directory.
-        let stdlib = if self.key.os.is_windows() {
+        let stdlib = if self.key.os().is_windows() {
             self.python_dir().join("Lib")
         } else {
             let lib_suffix = self.key.variant.suffix();
@@ -568,7 +595,12 @@ impl ManagedPythonInstallation {
     /// Ensure that the `sysconfig` data is patched to match the installation path.
     pub fn ensure_sysconfig_patched(&self) -> Result<(), Error> {
         if cfg!(unix) {
-            if *self.implementation() == ImplementationName::CPython {
+            if self.key.os().is_emscripten() {
+                // Emscripten's stdlib is a zip file so we can't update the
+                // sysconfig directly
+                return Ok(());
+            }
+            if self.implementation() == ImplementationName::CPython {
                 sysconfig::update_sysconfig(
                     self.path(),
                     self.key.major,
@@ -588,8 +620,8 @@ impl ManagedPythonInstallation {
     /// See <https://github.com/astral-sh/uv/issues/10598> for more information.
     pub fn ensure_dylib_patched(&self) -> Result<(), macos_dylib::Error> {
         if cfg!(target_os = "macos") {
-            if self.key().os.is_like_darwin() {
-                if *self.implementation() == ImplementationName::CPython {
+            if self.key().os().is_like_darwin() {
+                if self.implementation() == ImplementationName::CPython {
                     let dylib_path = self.python_dir().join("lib").join(format!(
                         "{}python{}{}{}",
                         std::env::consts::DLL_PREFIX,
@@ -604,6 +636,15 @@ impl ManagedPythonInstallation {
         Ok(())
     }
 
+    /// Ensure the build version is written to a BUILD file in the installation directory.
+    pub fn ensure_build_file(&self) -> Result<(), Error> {
+        if let Some(ref build) = self.build {
+            let build_file = self.path.join("BUILD");
+            fs::write(&build_file, build.as_ref())?;
+        }
+        Ok(())
+    }
+
     /// Returns `true` if the path is a link to this installation's binary, e.g., as created by
     /// [`create_bin_link`].
     pub fn is_bin_link(&self, path: &Path) -> bool {
@@ -627,7 +668,7 @@ impl ManagedPythonInstallation {
     }
 
     /// Returns `true` if self is a suitable upgrade of other.
-    pub fn is_upgrade_of(&self, other: &ManagedPythonInstallation) -> bool {
+    pub fn is_upgrade_of(&self, other: &Self) -> bool {
         // Require matching implementation
         if self.key.implementation != other.key.implementation {
             return false;
@@ -655,12 +696,12 @@ impl ManagedPythonInstallation {
         self.key.patch != other.key.patch
     }
 
-    pub fn url(&self) -> Option<&'static str> {
-        self.url
+    pub fn url(&self) -> Option<&str> {
+        self.url.as_deref()
     }
 
-    pub fn sha256(&self) -> Option<&'static str> {
-        self.sha256
+    pub fn sha256(&self) -> Option<&str> {
+        self.sha256.as_deref()
     }
 }
 
@@ -705,7 +746,7 @@ impl PythonMinorVersionLink {
     ) -> Option<Self> {
         let implementation = key.implementation();
         if !matches!(
-            implementation,
+            implementation.as_ref(),
             LenientImplementationName::Known(ImplementationName::CPython)
         ) {
             // We don't currently support transparent upgrades for PyPy or GraalPy.
@@ -744,7 +785,8 @@ impl PythonMinorVersionLink {
         let symlink_executable = executable_path_from_base(
             symlink_directory.as_path(),
             &executable_name.to_string_lossy(),
-            implementation,
+            &implementation,
+            *key.os(),
         );
         let minor_version_link = Self {
             symlink_directory,
@@ -764,7 +806,7 @@ impl PythonMinorVersionLink {
         installation: &ManagedPythonInstallation,
         preview: Preview,
     ) -> Option<Self> {
-        PythonMinorVersionLink::from_executable(
+        Self::from_executable(
             installation.executable(false).as_path(),
             installation.key(),
             preview,
@@ -828,18 +870,28 @@ fn executable_path_from_base(
     base: &Path,
     executable_name: &str,
     implementation: &LenientImplementationName,
+    os: Os,
 ) -> PathBuf {
-    if cfg!(unix)
+    if matches!(
+        implementation,
+        &LenientImplementationName::Known(ImplementationName::GraalPy)
+    ) {
+        // GraalPy is always in `bin/` regardless of the os
+        base.join("bin").join(executable_name)
+    } else if os.is_emscripten()
         || matches!(
             implementation,
-            &LenientImplementationName::Known(ImplementationName::GraalPy)
+            &LenientImplementationName::Known(ImplementationName::Pyodide)
         )
     {
-        base.join("bin").join(executable_name)
-    } else if cfg!(windows) {
+        // Emscripten's canonical executable is in the base directory
+        base.join(executable_name)
+    } else if os.is_windows() {
+        // On Windows, the executable is in the base directory
         base.join(executable_name)
     } else {
-        unimplemented!("Only Windows and Unix systems are supported.")
+        // On Unix, the executable is in `bin/`
+        base.join("bin").join(executable_name)
     }
 }
 
@@ -890,10 +942,7 @@ pub fn create_link_to_executable(link: &Path, executable: &Path) -> Result<(), E
 // TODO(zanieb): Only used in tests now.
 /// Generate a platform portion of a key from the environment.
 pub fn platform_key_from_env() -> Result<String, Error> {
-    let os = Os::from_env();
-    let arch = Arch::from_env();
-    let libc = Libc::from_env()?;
-    Ok(format!("{os}-{arch}-{libc}").to_lowercase())
+    Ok(Platform::from_env()?.to_string().to_lowercase())
 }
 
 impl fmt::Display for ManagedPythonInstallation {
