@@ -29,6 +29,7 @@ use uv_installer::{Plan, Planner, Preparer, SitePackages};
 use uv_normalize::PackageName;
 use uv_pep508::{MarkerEnvironment, RequirementOrigin};
 use uv_platform_tags::Tags;
+use uv_preview::Preview;
 use uv_pypi_types::{Conflicts, ResolverMarkerEnvironment};
 use uv_python::{PythonEnvironment, PythonInstallation};
 use uv_requirements::{
@@ -215,11 +216,10 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
                 build_dispatch.workspace_cache(),
             )
             .await
-            .map_err(|e| {
-                anyhow!(
-                    "Failed to read dependency groups from: {}\n{}",
-                    pyproject_path.display(),
-                    e
+            .with_context(|| {
+                format!(
+                    "Failed to read dependency groups from: {}",
+                    pyproject_path.display()
                 )
             })?;
 
@@ -389,6 +389,9 @@ pub(crate) struct Changelog {
 impl Changelog {
     /// Create a [`Changelog`] from a list of installed and uninstalled distributions.
     pub(crate) fn new(installed: Vec<CachedDist>, uninstalled: Vec<InstalledDist>) -> Self {
+        // SAFETY: This is allowed because `LocalDist` implements `Hash` and `Eq` based solely on
+        // the inner `kind`, and omits the types that rely on internal mutability.
+        #[allow(clippy::mutable_key_type)]
         let mut uninstalled: HashSet<_> = uninstalled.into_iter().map(LocalDist::from).collect();
 
         let (reinstalled, installed): (HashSet<_>, HashSet<_>) = installed
@@ -450,6 +453,7 @@ pub(crate) async fn install(
     installer_metadata: bool,
     dry_run: DryRun,
     printer: Printer,
+    preview: Preview,
 ) -> Result<Changelog, Error> {
     let start = std::time::Instant::now();
 
@@ -501,6 +505,132 @@ pub(crate) async fn install(
         return Ok(Changelog::default());
     }
 
+    // Partition into two sets: those that require build isolation, and those that disable it. This
+    // is effectively a heuristic to make `--no-build-isolation` work "more often" by way of giving
+    // `--no-build-isolation` packages "access" to the rest of the environment.
+    let (isolated_phase, shared_phase) = Plan {
+        cached,
+        remote,
+        reinstalls,
+        extraneous,
+    }
+    .partition(|name| build_dispatch.build_isolation().is_isolated(Some(name)));
+
+    let has_isolated_phase = !isolated_phase.is_empty();
+    let has_shared_phase = !shared_phase.is_empty();
+
+    let mut installs = vec![];
+    let mut uninstalls = vec![];
+
+    // Execute the isolated-build phase.
+    if has_isolated_phase {
+        let (isolated_installs, isolated_uninstalls) = execute_plan(
+            isolated_phase,
+            None,
+            resolution,
+            build_options,
+            link_mode,
+            hasher,
+            tags,
+            client,
+            in_flight,
+            concurrency,
+            build_dispatch,
+            cache,
+            venv,
+            logger.as_ref(),
+            installer_metadata,
+            printer,
+            preview,
+        )
+        .await?;
+        installs.extend(isolated_installs);
+        uninstalls.extend(isolated_uninstalls);
+    }
+
+    if has_shared_phase {
+        let (shared_installs, shared_uninstalls) = execute_plan(
+            shared_phase,
+            if has_isolated_phase {
+                Some(InstallPhase::Shared)
+            } else {
+                None
+            },
+            resolution,
+            build_options,
+            link_mode,
+            hasher,
+            tags,
+            client,
+            in_flight,
+            concurrency,
+            build_dispatch,
+            cache,
+            venv,
+            logger.as_ref(),
+            installer_metadata,
+            printer,
+            preview,
+        )
+        .await?;
+        installs.extend(shared_installs);
+        uninstalls.extend(shared_uninstalls);
+    }
+
+    if compile {
+        compile_bytecode(venv, &concurrency, cache, printer).await?;
+    }
+
+    // Construct a summary of the changes made to the environment.
+    let changelog = Changelog::new(installs, uninstalls);
+
+    // Notify the user of any environment modifications.
+    logger.on_complete(&changelog, printer)?;
+
+    Ok(changelog)
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+enum InstallPhase {
+    /// A dedicated phase for building and installing packages with build-isolation disabled.
+    Shared,
+}
+
+impl InstallPhase {
+    fn label(self) -> &'static str {
+        match self {
+            Self::Shared => "without build isolation",
+        }
+    }
+}
+
+/// Execute a [`Plan`] to install distributions into a Python environment.
+async fn execute_plan(
+    plan: Plan,
+    phase: Option<InstallPhase>,
+    resolution: &Resolution,
+    build_options: &BuildOptions,
+    link_mode: LinkMode,
+    hasher: &HashStrategy,
+    tags: &Tags,
+    client: &RegistryClient,
+    in_flight: &InFlight,
+    concurrency: Concurrency,
+    build_dispatch: &BuildDispatch<'_>,
+    cache: &Cache,
+    venv: &PythonEnvironment,
+    logger: &dyn InstallLogger,
+    installer_metadata: bool,
+    printer: Printer,
+    preview: Preview,
+) -> Result<(Vec<CachedDist>, Vec<InstalledDist>), Error> {
+    let Plan {
+        cached,
+        remote,
+        reinstalls,
+        extraneous,
+    } = plan;
+
     // Download, build, and unzip any missing distributions.
     let wheels = if remote.is_empty() {
         vec![]
@@ -522,7 +652,7 @@ pub(crate) async fn install(
             .prepare(remote.clone(), in_flight, resolution)
             .await?;
 
-        logger.on_prepare(wheels.len(), start, printer)?;
+        logger.on_prepare(wheels.len(), phase.map(InstallPhase::label), start, printer)?;
 
         wheels
     };
@@ -571,7 +701,7 @@ pub(crate) async fn install(
     let mut installs = wheels.into_iter().chain(cached).collect::<Vec<_>>();
     if !installs.is_empty() {
         let start = std::time::Instant::now();
-        installs = uv_installer::Installer::new(venv)
+        installs = uv_installer::Installer::new(venv, preview)
             .with_link_mode(link_mode)
             .with_cache(cache)
             .with_installer_metadata(installer_metadata)
@@ -586,17 +716,7 @@ pub(crate) async fn install(
         logger.on_install(installs.len(), start, printer)?;
     }
 
-    if compile {
-        compile_bytecode(venv, &concurrency, cache, printer).await?;
-    }
-
-    // Construct a summary of the changes made to the environment.
-    let changelog = Changelog::new(installs, uninstalls);
-
-    // Notify the user of any environment modifications.
-    logger.on_complete(&changelog, printer)?;
-
-    Ok(changelog)
+    Ok((installs, uninstalls))
 }
 
 /// Display a message about the interpreter that was selected for the operation.
@@ -858,10 +978,11 @@ pub(crate) fn diagnose_environment(
     resolution: &Resolution,
     venv: &PythonEnvironment,
     markers: &ResolverMarkerEnvironment,
+    tags: &Tags,
     printer: Printer,
 ) -> Result<(), Error> {
     let site_packages = SitePackages::from_environment(venv)?;
-    for diagnostic in site_packages.diagnostics(markers)? {
+    for diagnostic in site_packages.diagnostics(markers, tags)? {
         // Only surface diagnostics that are "relevant" to the current resolution.
         if resolution
             .distributions()
