@@ -18,11 +18,11 @@ use uv_cache_key::RepositoryUrl;
 use uv_client::{BaseClientBuilder, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
     Concurrency, Constraints, DependencyGroups, DependencyGroupsWithDefaults, DevMode, DryRun,
-    EditableMode, ExtrasSpecification, ExtrasSpecificationWithDefaults, InstallOptions, Preview,
-    PreviewFeatures, SourceStrategy,
+    EditableMode, ExtrasSpecification, ExtrasSpecificationWithDefaults, InstallOptions,
+    SourceStrategy,
 };
 use uv_dispatch::BuildDispatch;
-use uv_distribution::DistributionDatabase;
+use uv_distribution::{DistributionDatabase, LoweredExtraBuildDependencies};
 use uv_distribution_types::{
     Index, IndexName, IndexUrl, IndexUrls, NameRequirementSpecification, Requirement,
     RequirementSource, UnresolvedRequirement, VersionId,
@@ -30,8 +30,9 @@ use uv_distribution_types::{
 use uv_fs::{LockedFile, Simplified};
 use uv_git::GIT_STORE;
 use uv_git_types::GitReference;
-use uv_normalize::{DEV_DEPENDENCIES, DefaultExtras, DefaultGroups, PackageName};
-use uv_pep508::{ExtraName, MarkerTree, UnnamedRequirement, VersionOrUrl};
+use uv_normalize::{DEV_DEPENDENCIES, DefaultExtras, DefaultGroups, ExtraName, PackageName};
+use uv_pep508::{MarkerTree, UnnamedRequirement, VersionOrUrl};
+use uv_preview::{Preview, PreviewFeatures};
 use uv_pypi_types::{ParsedUrl, VerbatimParsedUrl};
 use uv_python::{Interpreter, PythonDownloads, PythonEnvironment, PythonPreference, PythonRequest};
 use uv_redacted::DisplaySafeUrl;
@@ -59,7 +60,7 @@ use crate::commands::project::{
 use crate::commands::reporters::{PythonDownloadReporter, ResolverReporter};
 use crate::commands::{ExitStatus, ScriptPath, diagnostics, project};
 use crate::printer::Printer;
-use crate::settings::{NetworkSettings, ResolverInstallerSettings};
+use crate::settings::ResolverInstallerSettings;
 
 /// Add one or more packages to the project requirements.
 #[allow(clippy::fn_params_excessive_bools)]
@@ -69,6 +70,9 @@ pub(crate) async fn add(
     frozen: bool,
     active: Option<bool>,
     no_sync: bool,
+    no_install_project: bool,
+    no_install_workspace: bool,
+    no_install_local: bool,
     requirements: Vec<RequirementsSource>,
     constraints: Vec<RequirementsSource>,
     marker: Option<MarkerTree>,
@@ -86,7 +90,7 @@ pub(crate) async fn add(
     workspace: Option<bool>,
     install_mirrors: PythonInstallMirrors,
     settings: ResolverInstallerSettings,
-    network_settings: NetworkSettings,
+    client_builder: BaseClientBuilder<'_>,
     script: Option<ScriptPath>,
     python_preference: PythonPreference,
     python_downloads: PythonDownloads,
@@ -187,12 +191,6 @@ pub(crate) async fn add(
             );
         }
 
-        let client_builder = BaseClientBuilder::new()
-            .retries_from_env()?
-            .connectivity(network_settings.connectivity)
-            .native_tls(network_settings.native_tls)
-            .allow_insecure_host(network_settings.allow_insecure_host.clone());
-
         // If we found a script, add to the existing metadata. Otherwise, create a new inline
         // metadata tag.
         let script = match script {
@@ -223,7 +221,7 @@ pub(crate) async fn add(
         let interpreter = ScriptInterpreter::discover(
             (&script).into(),
             python.as_deref().map(PythonRequest::parse),
-            &network_settings,
+            &client_builder,
             python_preference,
             python_downloads,
             &install_mirrors,
@@ -293,7 +291,7 @@ pub(crate) async fn add(
                 project_dir,
                 &defaulted_groups,
                 python.as_deref().map(PythonRequest::parse),
-                &network_settings,
+                &client_builder,
                 python_preference,
                 python_downloads,
                 &install_mirrors,
@@ -315,7 +313,7 @@ pub(crate) async fn add(
                 &defaulted_groups,
                 python.as_deref().map(PythonRequest::parse),
                 &install_mirrors,
-                &network_settings,
+                &client_builder,
                 python_preference,
                 python_downloads,
                 no_sync,
@@ -341,12 +339,9 @@ pub(crate) async fn add(
         })
         .ok();
 
-    let client_builder = BaseClientBuilder::new()
-        .retries_from_env()?
-        .connectivity(network_settings.connectivity)
-        .native_tls(network_settings.native_tls)
-        .keyring(settings.resolver.keyring_provider)
-        .allow_insecure_host(network_settings.allow_insecure_host.clone());
+    let client_builder = client_builder
+        .clone()
+        .keyring(settings.resolver.keyring_provider);
 
     // Read the requirements.
     let RequirementsSpecification {
@@ -395,11 +390,9 @@ pub(crate) async fn add(
             let hasher = HashStrategy::default();
             let sources = SourceStrategy::Enabled;
 
-            settings.resolver.index_locations.cache_index_credentials();
-
             // Initialize the registry client.
-            let client = RegistryClientBuilder::try_from(client_builder)?
-                .index_locations(&settings.resolver.index_locations)
+            let client = RegistryClientBuilder::new(client_builder.clone(), cache.clone())
+                .index_locations(settings.resolver.index_locations.clone())
                 .index_strategy(settings.resolver.index_strategy)
                 .markers(target.interpreter().markers())
                 .platform(target.interpreter().platform())
@@ -407,17 +400,16 @@ pub(crate) async fn add(
 
             // Determine whether to enable build isolation.
             let environment;
-            let build_isolation = if settings.resolver.no_build_isolation {
-                environment = PythonEnvironment::from_interpreter(target.interpreter().clone());
-                BuildIsolation::Shared(&environment)
-            } else if settings.resolver.no_build_isolation_package.is_empty() {
-                BuildIsolation::Isolated
-            } else {
-                environment = PythonEnvironment::from_interpreter(target.interpreter().clone());
-                BuildIsolation::SharedPackage(
-                    &environment,
-                    &settings.resolver.no_build_isolation_package,
-                )
+            let build_isolation = match &settings.resolver.build_isolation {
+                uv_configuration::BuildIsolation::Isolate => BuildIsolation::Isolated,
+                uv_configuration::BuildIsolation::Shared => {
+                    environment = PythonEnvironment::from_interpreter(target.interpreter().clone());
+                    BuildIsolation::Shared(&environment)
+                }
+                uv_configuration::BuildIsolation::SharedPackage(packages) => {
+                    environment = PythonEnvironment::from_interpreter(target.interpreter().clone());
+                    BuildIsolation::SharedPackage(&environment, packages)
+                }
             };
 
             // Resolve the flat indexes from `--find-links`.
@@ -436,23 +428,27 @@ pub(crate) async fn add(
                 FlatIndex::from_entries(entries, None, &hasher, &settings.resolver.build_options)
             };
 
-            // Create a build dispatch.
+            // Lower the extra build dependencies, if any.
             let extra_build_requires = if let AddTarget::Project(project, _) = &target {
-                uv_distribution::ExtraBuildRequires::from_workspace(
+                LoweredExtraBuildDependencies::from_workspace(
                     settings.resolver.extra_build_dependencies.clone(),
                     project.workspace(),
                     &settings.resolver.index_locations,
                     settings.resolver.sources,
                 )?
             } else {
-                uv_distribution::ExtraBuildRequires::from_lowered(
+                LoweredExtraBuildDependencies::from_non_lowered(
                     settings.resolver.extra_build_dependencies.clone(),
                 )
-            };
+            }
+            .into_inner();
+
+            // Create a build dispatch.
+            let extra_build_variables = settings.resolver.extra_build_variables.clone();
             let build_dispatch = BuildDispatch::new(
                 &client,
                 cache,
-                build_constraints,
+                &build_constraints,
                 target.interpreter(),
                 &settings.resolver.index_locations,
                 &flat_index,
@@ -463,6 +459,7 @@ pub(crate) async fn add(
                 &settings.resolver.config_settings_package,
                 build_isolation,
                 &extra_build_requires,
+                &extra_build_variables,
                 settings.resolver.link_mode,
                 &settings.resolver.build_options,
                 &build_hasher,
@@ -730,13 +727,16 @@ pub(crate) async fn add(
         lock_state,
         sync_state,
         locked,
+        no_install_project,
+        no_install_workspace,
+        no_install_local,
         &defaulted_extras,
         &defaulted_groups,
         raw,
         bounds,
         constraints,
         &settings,
-        &network_settings,
+        &client_builder,
         installer_metadata,
         concurrency,
         cache,
@@ -751,7 +751,7 @@ pub(crate) async fn add(
                 let _ = snapshot.revert();
             }
             match err {
-                ProjectError::Operation(err) => diagnostics::OperationDiagnostic::native_tls(network_settings.native_tls).with_hint(format!("If you want to add the package regardless of the failed resolution, provide the `{}` flag to skip locking and syncing.", "--frozen".green()))
+                ProjectError::Operation(err) => diagnostics::OperationDiagnostic::native_tls(client_builder.is_native_tls()).with_hint(format!("If you want to add the package regardless of the failed resolution, provide the `{}` flag to skip locking and syncing.", "--frozen".green()))
                     .report(err)
                     .map_or(Ok(ExitStatus::Failure), |err| Err(err.into())),
                 err => Err(err.into()),
@@ -958,13 +958,16 @@ async fn lock_and_sync(
     lock_state: UniversalState,
     sync_state: PlatformState,
     locked: bool,
+    no_install_project: bool,
+    no_install_workspace: bool,
+    no_install_local: bool,
     extras: &ExtrasSpecificationWithDefaults,
     groups: &DependencyGroupsWithDefaults,
     raw: bool,
     bound_kind: Option<AddBoundsKind>,
     constraints: Vec<NameRequirementSpecification>,
     settings: &ResolverInstallerSettings,
-    network_settings: &NetworkSettings,
+    client_builder: &BaseClientBuilder<'_>,
     installer_metadata: bool,
     concurrency: Concurrency,
     cache: &Cache,
@@ -978,7 +981,7 @@ async fn lock_and_sync(
             LockMode::Write(target.interpreter())
         },
         &settings.resolver,
-        network_settings,
+        client_builder,
         &lock_state,
         Box::new(DefaultResolveLogger),
         concurrency,
@@ -1100,7 +1103,7 @@ async fn lock_and_sync(
                     LockMode::Write(target.interpreter())
                 },
                 &settings.resolver,
-                network_settings,
+                client_builder,
                 &lock_state,
                 Box::new(SummaryResolveLogger),
                 concurrency,
@@ -1144,11 +1147,16 @@ async fn lock_and_sync(
         extras,
         groups,
         EditableMode::Editable,
-        InstallOptions::default(),
+        InstallOptions::new(
+            no_install_project,
+            no_install_workspace,
+            no_install_local,
+            vec![],
+        ),
         Modifications::Sufficient,
         None,
         settings.into(),
-        network_settings,
+        client_builder,
         &sync_state,
         Box::new(DefaultInstallLogger),
         installer_metadata,
