@@ -7,13 +7,14 @@ use console::Term;
 use owo_colors::{AnsiColors, OwoColorize};
 use tokio::sync::Semaphore;
 use tracing::{debug, info, trace};
-use uv_auth::Credentials;
+use uv_auth::{Credentials, PyxTokenStore};
 use uv_cache::Cache;
 use uv_client::{AuthIntegration, BaseClient, BaseClientBuilder, RegistryClientBuilder};
 use uv_configuration::{KeyringProviderType, TrustedPublishing};
-use uv_distribution_types::{Index, IndexCapabilities, IndexLocations, IndexUrl};
+use uv_distribution_types::{IndexCapabilities, IndexLocations, IndexUrl};
 use uv_publish::{
-    CheckUrlClient, TrustedPublishResult, check_trusted_publishing, files_for_publishing, upload,
+    CheckUrlClient, FormMetadata, PublishError, TrustedPublishResult, check_trusted_publishing,
+    files_for_publishing, upload,
 };
 use uv_redacted::DisplaySafeUrl;
 use uv_warnings::{warn_user_once, write_error_chain};
@@ -21,29 +22,42 @@ use uv_warnings::{warn_user_once, write_error_chain};
 use crate::commands::reporters::PublishReporter;
 use crate::commands::{ExitStatus, human_readable_bytes};
 use crate::printer::Printer;
-use crate::settings::NetworkSettings;
 
 pub(crate) async fn publish(
     paths: Vec<String>,
     publish_url: DisplaySafeUrl,
     trusted_publishing: TrustedPublishing,
     keyring_provider: KeyringProviderType,
-    network_settings: &NetworkSettings,
+    client_builder: &BaseClientBuilder<'_>,
     username: Option<String>,
     password: Option<String>,
     check_url: Option<IndexUrl>,
+    index_locations: IndexLocations,
+    dry_run: bool,
     cache: &Cache,
     printer: Printer,
 ) -> Result<ExitStatus> {
-    if network_settings.connectivity.is_offline() {
+    if client_builder.is_offline() {
         bail!("Unable to publish files in offline mode");
     }
 
     let files = files_for_publishing(paths)?;
     match files.len() {
         0 => bail!("No files found to publish"),
-        1 => writeln!(printer.stderr(), "Publishing 1 file to {publish_url}")?,
-        n => writeln!(printer.stderr(), "Publishing {n} files {publish_url}")?,
+        1 => {
+            if dry_run {
+                writeln!(printer.stderr(), "Checking 1 file against {publish_url}")?;
+            } else {
+                writeln!(printer.stderr(), "Publishing 1 file to {publish_url}")?;
+            }
+        }
+        n => {
+            if dry_run {
+                writeln!(printer.stderr(), "Checking {n} files against {publish_url}")?;
+            } else {
+                writeln!(printer.stderr(), "Publishing {n} files to {publish_url}")?;
+            }
+        }
     }
 
     // * For the uploads themselves, we roll our own retries due to
@@ -56,18 +70,18 @@ pub(crate) async fn publish(
     //   shouldn't try cloning the request to make an unauthenticated request first, but we want
     //   keyring integration. For trusted publishing, we use an OIDC auth routine without keyring
     //   or other auth integration.
-    let upload_client = BaseClientBuilder::new()
+    let upload_client = client_builder
+        .clone()
         .retries(0)
         .keyring(keyring_provider)
-        .native_tls(network_settings.native_tls)
-        .allow_insecure_host(network_settings.allow_insecure_host.clone())
         // Don't try cloning the request to make an unauthenticated request first.
         .auth_integration(AuthIntegration::OnlyAuthenticated)
         // Set a very high timeout for uploads, connections are often 10x slower on upload than
         // download. 15 min is taken from the time a trusted publishing token is valid.
         .default_timeout(Duration::from_secs(15 * 60))
         .build();
-    let oidc_client = BaseClientBuilder::new()
+    let oidc_client = client_builder
+        .clone()
         .auth_integration(AuthIntegration::NoAuthMiddleware)
         .wrap_existing(&upload_client);
     // We're only checking a single URL and one at a time, so 1 permit is sufficient
@@ -88,18 +102,10 @@ pub(crate) async fn publish(
 
     // Initialize the registry client.
     let check_url_client = if let Some(index_url) = &check_url {
-        let index_locations = IndexLocations::new(
-            vec![Index::from_index_url(index_url.clone())],
-            Vec::new(),
-            false,
-        );
-        let registry_client_builder = RegistryClientBuilder::new(cache.clone())
-            .retries_from_env()?
-            .native_tls(network_settings.native_tls)
-            .connectivity(network_settings.connectivity)
-            .allow_insecure_host(network_settings.allow_insecure_host.clone())
-            .index_locations(&index_locations)
-            .keyring(keyring_provider);
+        let registry_client_builder =
+            RegistryClientBuilder::new(client_builder.clone(), cache.clone())
+                .index_locations(index_locations)
+                .keyring(keyring_provider);
         Some(CheckUrlClient {
             index_url: index_url.clone(),
             registry_client_builder,
@@ -123,15 +129,48 @@ pub(crate) async fn publish(
 
         let size = fs_err::metadata(&file)?.len();
         let (bytes, unit) = human_readable_bytes(size);
-        writeln!(
-            printer.stderr(),
-            "{} {filename} {}",
-            "Uploading".bold().green(),
-            format!("({bytes:.1}{unit})").dimmed()
-        )?;
+        if dry_run {
+            writeln!(
+                printer.stderr(),
+                "{} {filename} {}",
+                "Checking".bold().cyan(),
+                format!("({bytes:.1}{unit})").dimmed()
+            )?;
+        } else {
+            writeln!(
+                printer.stderr(),
+                "{} {filename} {}",
+                "Uploading".bold().green(),
+                format!("({bytes:.1}{unit})").dimmed()
+            )?;
+        }
+
+        // Collect the metadata for the file.
+        let form_metadata = FormMetadata::read_from_file(&file, &filename)
+            .await
+            .map_err(|err| PublishError::PublishPrepare(file.clone(), Box::new(err)))?;
+
+        // Run validation checks on the file, but don't upload it (if possible).
+        let store = PyxTokenStore::from_settings()?;
+        uv_publish::validate(
+            &file,
+            &form_metadata,
+            &raw_filename,
+            &publish_url,
+            &store,
+            &upload_client,
+            &credentials,
+        )
+        .await?;
+
+        if dry_run {
+            continue;
+        }
+
         let reporter = PublishReporter::single(printer);
         let uploaded = upload(
             &file,
+            &form_metadata,
             &raw_filename,
             &filename,
             &publish_url,
@@ -144,6 +183,7 @@ pub(crate) async fn publish(
         )
         .await?; // Filename and/or URL are already attached, if applicable.
         info!("Upload succeeded");
+
         if !uploaded {
             writeln!(
                 printer.stderr(),
@@ -288,11 +328,11 @@ async fn gather_credentials(
 
     // If applicable, fetch the password from the keyring eagerly to avoid user confusion about
     // missing keyring entries later.
-    if let Some(keyring_provider) = keyring_provider.to_provider() {
+    if let Some(provider) = keyring_provider.to_provider() {
         if password.is_none() {
             if let Some(username) = &username {
                 debug!("Fetching password from keyring");
-                if let Some(keyring_password) = keyring_provider
+                if let Some(keyring_password) = provider
                     .fetch(DisplaySafeUrl::ref_cast(&publish_url), Some(username))
                     .await
                     .as_ref()
@@ -347,7 +387,7 @@ mod tests {
         username: Option<String>,
         password: Option<String>,
     ) -> Result<(DisplaySafeUrl, Credentials)> {
-        let client = BaseClientBuilder::new().build();
+        let client = BaseClientBuilder::default().build();
         gather_credentials(
             url,
             username,
