@@ -11,12 +11,14 @@ use futures::stream::FuturesUnordered;
 use indexmap::IndexSet;
 use itertools::{Either, Itertools};
 use owo_colors::{AnsiColors, OwoColorize};
+use reqwest_retry::policies::ExponentialBackoff;
 use rustc_hash::{FxHashMap, FxHashSet};
 use tracing::{debug, trace};
 
-use uv_configuration::{Preview, PreviewFeatures};
+use uv_client::{BaseClientBuilder, retries_from_env};
 use uv_fs::Simplified;
 use uv_platform::{Arch, Libc};
+use uv_preview::{Preview, PreviewFeatures};
 use uv_python::downloads::{
     self, ArchRequest, DownloadResult, ManagedPythonDownload, PythonDownloadRequest,
 };
@@ -36,7 +38,6 @@ use crate::commands::python::{ChangeEvent, ChangeEventKind};
 use crate::commands::reporters::PythonDownloadReporter;
 use crate::commands::{ExitStatus, elapsed};
 use crate::printer::Printer;
-use crate::settings::NetworkSettings;
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 struct InstallRequest {
@@ -96,7 +97,13 @@ impl InstallRequest {
 
 impl std::fmt::Display for InstallRequest {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{}", self.request)
+        let request = self.request.to_canonical_string();
+        let download = self.download_request.to_string();
+        if request != download {
+            write!(f, "{request} ({download})")
+        } else {
+            write!(f, "{request}")
+        }
     }
 }
 
@@ -157,7 +164,7 @@ pub(crate) async fn install(
     python_install_mirror: Option<String>,
     pypy_install_mirror: Option<String>,
     python_downloads_json_url: Option<String>,
-    network_settings: NetworkSettings,
+    client_builder: BaseClientBuilder<'_>,
     default: bool,
     python_downloads: PythonDownloads,
     no_config: bool,
@@ -223,6 +230,12 @@ pub(crate) async fn install(
                     .with_preference(VersionFilePreference::Versions),
             )
             .await?
+            .inspect(|file| {
+                debug!(
+                    "Found Python version file at: {}",
+                    file.path().user_display()
+                );
+            })
             .map(PythonVersionFile::into_versions)
             .unwrap_or_else(|| {
                 // If no version file is found and no requests were made
@@ -237,14 +250,14 @@ pub(crate) async fn install(
                 }]
             })
             .into_iter()
-            .map(|a| InstallRequest::new(a, python_downloads_json_url.as_deref()))
+            .map(|request| InstallRequest::new(request, python_downloads_json_url.as_deref()))
             .collect::<Result<Vec<_>>>()?
         }
     } else {
         targets
             .iter()
             .map(|target| PythonRequest::parse(target.as_str()))
-            .map(|a| InstallRequest::new(a, python_downloads_json_url.as_deref()))
+            .map(|request| InstallRequest::new(request, python_downloads_json_url.as_deref()))
             .collect::<Result<Vec<_>>>()?
     };
 
@@ -390,13 +403,11 @@ pub(crate) async fn install(
         .unique_by(|download| download.key())
         .collect::<Vec<_>>();
 
-    // Download and unpack the Python versions concurrently
-    let client = uv_client::BaseClientBuilder::new()
-        .retries_from_env()?
-        .connectivity(network_settings.connectivity)
-        .native_tls(network_settings.native_tls)
-        .allow_insecure_host(network_settings.allow_insecure_host.clone())
-        .build();
+    // Python downloads are performing their own retries to catch stream errors, disable the
+    // default retries to avoid the middleware from performing uncontrolled retries.
+    let retry_policy = ExponentialBackoff::builder().build_with_max_retries(retries_from_env()?);
+    let client = client_builder.retries(0).build();
+
     let reporter = PythonDownloadReporter::new(printer, downloads.len() as u64);
     let mut tasks = FuturesUnordered::new();
 
@@ -407,6 +418,7 @@ pub(crate) async fn install(
                 download
                     .fetch_with_retry(
                         &client,
+                        &retry_policy,
                         installations_dir,
                         &scratch_dir,
                         reinstall,
@@ -471,6 +483,7 @@ pub(crate) async fn install(
         installation.ensure_externally_managed()?;
         installation.ensure_sysconfig_patched()?;
         installation.ensure_canonical_executables()?;
+        installation.ensure_build_file()?;
         if let Err(e) = installation.ensure_dylib_patched() {
             e.warn_user(installation);
         }
@@ -565,10 +578,14 @@ pub(crate) async fn install(
 
     if !changelog.installed.is_empty() {
         for install_key in &changelog.installed {
-            // Make a note if the selected python is non-native for the architecture,
-            // if none of the matching user requests were explicit
+            // Make a note if the selected python is non-native for the architecture, if none of the
+            // matching user requests were explicit.
+            //
+            // Emscripten is exempted as it is always "emulated".
             let native_arch = Arch::from_env();
-            if install_key.arch().family() != native_arch.family() {
+            if install_key.arch().family() != native_arch.family()
+                && !install_key.os().is_emscripten()
+            {
                 let not_explicit =
                     requests_by_new_installation
                         .get(install_key)
@@ -706,6 +723,7 @@ pub(crate) async fn install(
                         Some(true) => ("error", AnsiColors::Red),
                     };
 
+                    trace!("Error trace: {err:?}");
                     write_error_chain(
                         err.context(format!("Failed to create registry entry for {key}"))
                             .as_ref(),
@@ -763,6 +781,9 @@ fn create_bin_links(
 
     for target in targets {
         let target = bin.join(target);
+        if upgrade && !target.try_exists().unwrap_or_default() {
+            continue;
+        }
         let executable = if upgradeable {
             if let Some(minor_version_link) =
                 PythonMinorVersionLink::from_installation(installation, preview)
